/*
* Licensee agrees that the example code provided to Licensee has been developed and released by Bosch solely as an example to be used as a potential reference for Licensee�s application development. 
* Fitness and suitability of the example code for any use within Licensee�s applications need to be verified by Licensee on its own authority by taking appropriate state of the art actions and measures (e.g. by means of quality assurance measures).
* Licensee shall be responsible for conducting the development of its applications as well as integration of parts of the example code into such applications, taking into account the state of the art of technology and any statutory regulations and provisions applicable for such applications. Compliance with the functional system requirements and testing there of (including validation of information/data security aspects and functional safety) and release shall be solely incumbent upon Licensee. 
* For the avoidance of doubt, Licensee shall be responsible and fully liable for the applications and any distribution of such applications into the market.
* 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 
*     (1) Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer. 
* 
*     (2) Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.  
*     
*     (3)The name of the author may not be used to
*     endorse or promote products derived from this software without
*     specific prior written permission.
* 
*  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
*  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
*  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
*  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
*  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
*  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
*  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
*  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
*  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
*  POSSIBILITY OF SUCH DAMAGE.
*/
/*----------------------------------------------------------------------------*/
/**
 * @ingroup APPS_LIST
 *
 * @defgroup XDK_APPLICATION_TEMPLATE XdkApplicationTemplate
 * @{
 *
 * @brief XDK Application Template
 *
 * @details XDK Application Template without any functionality.
 * Could be used as a starting point to develop new application based on XDK platform.
 *
 * @file
 **/
/* module includes ********************************************************** */

/* own header files */
#include "XdkAppInfo.h"
#undef BCDS_MODULE_ID  /* Module ID define before including Basics package*/
#define BCDS_MODULE_ID XDK_APP_MODULE_ID_APP_CONTROLLER

/* own header files */
#include "AppController.h"

/* system header files */
#include <stdio.h>

/* additional interface header files */
#include "XDK_LED.h"
#include "XDK_Button.h"
#include "BCDS_CmdProcessor.h"
#include "FreeRTOS.h"
#include "task.h"
#include "simplelink.h"
#include "BCDS_WlanConnect.h"
#include "BCDS_BSP_LED.h"
#include "BSP_BoardType.h"

/* Own header files */
#include "host_programming_1.0.1.6-2.6.0.5_ucf.h"
#include "host_programming_1.0.1.6-2.6.0.5_ucf-signed.h"
#include "host_programming_1.0.1.6-2.7.0.0_ucf.h"
#include "host_programming_1.0.1.6-2.7.0.0_ucf-signed.h"
#include "host_programming_1.0.1.11-2.9.0.0_ucf.h"
#include "host_programming_1.0.1.11-2.9.0.0_ucf-signed.h"
#include "host_programming_1.0.1.11-2.10.0.0_ucf.h"
#include "host_programming_1.0.1.11-2.10.0.0_ucf-signed.h"
#include "host_programming_1.0.1.13-2.11.0.1_ucf.h"
#include "host_programming_1.0.1.13-2.11.0.1_ucf-signed.h"
#include "host_programming_1.0.1.15-2.14.0.0_ucf.h"
#include "host_programming_1.0.1.15-2.14.0.0_ucf-signed.h"
#include "dummyCertificate.h"

/* constant definitions ***************************************************** */
#define VERSION_OFFSET      (8)
#define UCF_OFFSET      	(20)
#define CHUNK_LEN			(1024)
#define LEN_128KB			(131072)
#define FILE_NAME           "/cert/ca.pem"
#define NUM_OF_SP			6

#define find_min(a,b) (((a) < (b)) ? (a) : (b))

/* local variables ********************************************************** */
static CmdProcessor_T * AppCmdProcessor;/**< Handle to store the main Command processor handle to be used by run-time event driven threads */

static void Button1Callback(ButtonEvent_T buttonEvent);
static void Button2Callback(ButtonEvent_T buttonEvent);

static Button_Setup_T ButtonSetup =
        {
                .CmdProcessorHandle = NULL,
                .InternalButton1isEnabled = true,
                .InternalButton2isEnabled = true,
                .InternalButton1Callback = Button1Callback,
                .InternalButton2Callback = Button2Callback,
        };/**< Button setup parameters */

char printBuffer[1024];

/* global variables ********************************************************* */

/* inline functions ********************************************************* */

/* local functions ********************************************************** */
static int WiFiHostProgramming(void);
static void UploadCertificate(void);
static void ReadVersion(void);

static void ReadVersion(void)
{
    SlVersionFull ver;
    _u8 pConfigOpt, pConfigLen;
    _i32 retVal = 0;

    /* read the version and print it on terminal */
    pConfigOpt = SL_DEVICE_GENERAL_VERSION;
    pConfigLen = sizeof(SlVersionFull);
    retVal = sl_DevGet(SL_DEVICE_GENERAL_CONFIGURATION, &pConfigOpt, &pConfigLen, (_u8 *) (&ver));

    if (retVal < 0)
    {
        printf( "\r\nReading version failed. Error code: %d\r\n", (int) retVal);
    }

    if (ver.ChipFwAndPhyVersion.ChipId & 0x10)
        printf("\r\nThis is a CC3200\r\n");
    else
        printf("\r\nThis is a CC3100\r\n");

    if (ver.ChipFwAndPhyVersion.ChipId & 0x2)
        printf("Z device\r\n");
    else
        printf("R device\r\n");

    sprintf(printBuffer, "NWP %d.%d.%d.%d\r\nMAC 31.%d.%d.%d.%d\n\rPHY %d.%d.%d.%d\r\n\r\n",
            (_u8) ver.NwpVersion[0], (_u8) ver.NwpVersion[1], (_u8) ver.NwpVersion[2], (_u8) ver.NwpVersion[3],
            (_u8) ver.ChipFwAndPhyVersion.FwVersion[0], (_u8) ver.ChipFwAndPhyVersion.FwVersion[1],
            (_u8) ver.ChipFwAndPhyVersion.FwVersion[2], (_u8) ver.ChipFwAndPhyVersion.FwVersion[3],
            ver.ChipFwAndPhyVersion.PhyVersion[0], (_u8) ver.ChipFwAndPhyVersion.PhyVersion[1],
            ver.ChipFwAndPhyVersion.PhyVersion[2], (_u8) ver.ChipFwAndPhyVersion.PhyVersion[3]);

    printf(printBuffer);
}

static int WiFiHostProgramming(void)
{
    _i32 fileHandle = -1;

    _u32 Token = 0;
    _i32 retVal = 0;
    _u32 remainingLen, movingOffset, chunkLen, index;
    const unsigned char *binary[NUM_OF_SP] = {servicePackImage26, servicePackImage27, servicePackImage29, servicePackImage210, servicePackImage211, servicepack_1_0_1_15_2_14_0_0_ucf};
    unsigned int binaryLen[NUM_OF_SP] = {servicePackImageLen26, servicePackImageLen27, servicePackImageLen29, servicePackImageLen210, servicePackImageLen211, servicepack_1_0_1_15_2_14_0_0_ucf_len};
    const unsigned char *binarySig[NUM_OF_SP] = {servicePackImageSig26, servicePackImageSig27, servicePackImageSig29, servicePackImageSig210, servicePackImageSig211, servicepack_1_0_1_15_2_14_0_0_ucf_signed_bin };
    unsigned int binarySigLen[NUM_OF_SP] = {servicePackImageSigLen26, servicePackImageSigLen27, servicePackImageSigLen29, servicePackImageSigLen210, servicePackImageSigLen211,servicepack_1_0_1_15_2_14_0_0_ucf_signed_bin_len };

#ifdef FORMAT_ENABLE
    /* Initialize the bootloader UART interface */
    bl_uart_Configure();

    /* Configuring CC3100 nHib pin */
    P1SEL &= ~BIT6;
    P1OUT &= ~BIT6;
    P1DIR |= BIT6;

    printf((_u8 *)"\r\nSetting a break signal and resetting the device");
    /* setting a break signal prior to reset in order to enter boot loader mode */
    bl_uart_SetBreak();

    sl_DeviceDisable();

    Delay(100);

    sl_DeviceEnable();

    bl_read_ack();

    /* Initialize the bootloader UART interface - this would flush the RX lines*/
    bl_uart_Configure();

    flashSize = size1MB;
    printf((_u8 *)"\r\nFormatting the device to 1MB");
    bl_send_format(flashSize);

    bl_read_ack();

    printf((_u8 *)"\r\nDisconnecting from the device");
    bl_send_disconnect();

    bl_read_ack();
#endif	/* FORMAT_ENABLE */

    /* Reading the Device version details before flashing the CC3100 module*/
    ReadVersion();

    for (index = 0; index < NUM_OF_SP; index++) {
		/* create/open the servicepack file for 128KB with rollback, secured and public write */
		printf( "\r\nOpenning ServicePack file\r\n");
		retVal = sl_FsOpen((_u8 *)"/sys/servicepack.ucf",
				FS_MODE_OPEN_CREATE(LEN_128KB, _FS_FILE_OPEN_FLAG_SECURE | _FS_FILE_OPEN_FLAG_COMMIT | _FS_FILE_PUBLIC_WRITE),
				&Token, &fileHandle);
		if (retVal < 0)
		{
			sprintf(printBuffer, "Cannot open ServicePack file. Error code: %d\r\n", (int) retVal);
			printf(printBuffer);
			retVal = BSP_LED_Switch((uint32_t) BSP_XDK_LED_R, (uint32_t) BSP_LED_COMMAND_ON);
			return -1;
		}

		/* program the servicepack */
		printf("Programming ServicePack file\r\n");

		remainingLen = binaryLen[index];
		movingOffset = 0;
		chunkLen = (_u32) find_min(CHUNK_LEN, remainingLen);
		printf("rem = %ld    off = %ld    bin = %d    chunk = %ld\r\n", remainingLen, movingOffset, binary[index][movingOffset], chunkLen);

		/* Flashing must be done in 1024 bytes chunks */
		do
		{
			retVal = sl_FsWrite(fileHandle, movingOffset, (_u8 *) &binary[index][movingOffset], chunkLen);
			if (retVal < 0)
			{
				sprintf(printBuffer, "Cannot program ServicePack file. Error code: %d\r\n", (int) retVal);
				printf(printBuffer);
				retVal = BSP_LED_Switch((uint32_t) BSP_XDK_LED_R, (uint32_t) BSP_LED_COMMAND_ON);
				return -1;
			}

			remainingLen -= chunkLen;
			movingOffset += chunkLen;
			chunkLen = (_u32) find_min(CHUNK_LEN, remainingLen);
			printf("rem = %ld    off = %ld    bin = %d    chunk = %ld\r\n", remainingLen, movingOffset, binary[index][movingOffset], chunkLen);
		} while (chunkLen > 0);

		/* close the servicepack file */
		printf("Closing ServicePack file\r\n");
		retVal = sl_FsClose(fileHandle, 0, (_u8 *) binarySig[index], binarySigLen[index]);

		if (retVal < 0)
		{
			sprintf(printBuffer, "Cannot close ServicePack file. Error code: %d\r\n", (int) retVal);
			printf(printBuffer);
			retVal = BSP_LED_Switch((uint32_t) BSP_XDK_LED_R, (uint32_t) BSP_LED_COMMAND_ON);
			return -1;
		}

		printf("ServicePack successfuly programmed\r\n");

		/* Turn On the Yellow LED */
		retVal = BSP_LED_Switch((uint32_t) BSP_XDK_LED_Y, (uint32_t) BSP_LED_COMMAND_ON);

		printf("Restarting CC3100... \r\n");
		/* Stop the CC3100 device */
		sl_Stop(0xFF);

		vTaskDelay(2000);
		/* Initializing the CC3100 device */
		sl_Start(0, 0, 0);
		/* Reading the Device version details after flashing the CC3100 module*/
		ReadVersion();
		vTaskDelay(1000);
    }

    return 0;
}

static void UploadCertificate(void)
{
    _i32 fileHandle = -1;
    _u32 token = 0;
    _i32 retVal;
    _u32 remainingLen, movingOffset, chunkLen;

    printf("writing certificate %s\n", FILE_NAME);

    retVal = sl_FsOpen((_u8 *) FILE_NAME,
            FS_MODE_OPEN_CREATE(LEN_128KB, _FS_FILE_OPEN_FLAG_COMMIT | _FS_FILE_PUBLIC_WRITE),
            &token, &fileHandle);
    if (retVal < 0)
    {
        //open failed
        retVal = BSP_LED_Switch((uint32_t) BSP_XDK_LED_R, (uint32_t) BSP_LED_COMMAND_ON);
        return;
    }

    /* program the Certificate */

    remainingLen = sizeof(ca_pem);
    movingOffset = 0;
    chunkLen = (_u32) find_min(CHUNK_LEN, remainingLen);

    printf("Programming certificate file\n");

    /* Flashing must be done in 1024 bytes chunks */
    do
    {
        retVal = sl_FsWrite(fileHandle, movingOffset, (_u8 *) &ca_pem[movingOffset], chunkLen);
        if (retVal < 0)
        {
            //write failed
            retVal = BSP_LED_Switch((uint32_t) BSP_XDK_LED_Y, (uint32_t) BSP_LED_COMMAND_ON);
            return;
        }

        remainingLen -= chunkLen;
        movingOffset += chunkLen;
        chunkLen = (_u32) find_min(CHUNK_LEN, remainingLen);
    } while (chunkLen > 0);

    /* close the certificateImage file */
    printf("Closing certificateImage file\n");
    retVal = sl_FsClose(fileHandle, 0, 0, 0);

    if (retVal < 0)
    {
        //close failed
        retVal = BSP_LED_Switch((uint32_t) BSP_XDK_LED_O, (uint32_t) BSP_LED_COMMAND_ON);
        return;
    }

    printf("Certificate successfuly programmed\n");

    /* Turn On the Yellow LED */
    retVal = BSP_LED_Switch((uint32_t) BSP_XDK_LED_Y, (uint32_t) BSP_LED_COMMAND_ON);

    printf("Restarting CC3100... \n");
    /* Stop the CC3100 device */
    sl_Stop(0xFF);

    vTaskDelay(1000);
    /* Initializing the CC3100 device */
    sl_Start(0, 0, 0);

    printf("file write complete\n");
}

/* Process the Button1 Data in side the Application context */
static void Button1Callback(ButtonEvent_T buttonEvent)
{
    switch (buttonEvent)
    {
    case BUTTON_EVENT_PRESSED:
        BSP_LED_SwitchAll(BSP_LED_COMMAND_OFF);
        break;
    case BUTTON_EVENT_RELEASED:
        ReadVersion();
        break;
    default:
        printf("FATAL Error:Unsolicited button event occurred for PB1 \n\r");
        break;
    }
}

/* Process the Button2 Data inside the Application context */
static void Button2Callback(ButtonEvent_T buttonEvent)
{
    switch (buttonEvent)
    {
        case BUTTON_EVENT_PRESSED:
            BSP_LED_SwitchAll(BSP_LED_COMMAND_OFF);
            break;

        case BUTTON_EVENT_RELEASED:
            WiFiHostProgramming();
            vTaskDelay(2000);
            UploadCertificate();
            break;
        default:
            printf("FATAL Error:Unsolicited button event occurred for PB2 \n\r");
            break;
    }
}

/**
 * @brief To enable the necessary modules for the application
 *
 * @param [in] param1
 * A generic pointer to any context data structure which will be passed to the function when it is invoked by the command processor.
 *
 * @param [in] param2
 * A generic 32 bit value  which will be passed to the function when it is invoked by the command processor..
 */
static void AppControllerEnable(void * param1, uint32_t param2)
{
    BCDS_UNUSED(param1);
    BCDS_UNUSED(param2);
    Retcode_T retcode = RETCODE_OK;

    retcode = LED_Enable();
    if (RETCODE_OK == retcode)
    {
        retcode = Button_Enable();
    }
    /* Initializing the CC3100 device */
    if (RETCODE_OK == retcode)
    {
    	retcode = WlanConnect_Init();
    }
    if (RETCODE_OK != retcode)
    {
        printf("AppControllerEnable : Failed \r\n");
        Retcode_RaiseError(retcode);
        assert(0); /* To provide LED indication for the user */
    }
}

/**
 * @brief To setup the necessary modules for the application
 *
 * @param [in] param1
 * A generic pointer to any context data structure which will be passed to the function when it is invoked by the command processor.
 *
 * @param [in] param2
 * A generic 32 bit value  which will be passed to the function when it is invoked by the command processor..
 */
static void AppControllerSetup(void * param1, uint32_t param2)
{
    BCDS_UNUSED(param1);
    BCDS_UNUSED(param2);
    Retcode_T retcode = RETCODE_OK;

    retcode = LED_Setup();
    if (RETCODE_OK == retcode)
    {
        ButtonSetup.CmdProcessorHandle = AppCmdProcessor;
        retcode = Button_Setup(&ButtonSetup);
    }
    if (RETCODE_OK == retcode)
    {

        retcode = CmdProcessor_Enqueue(AppCmdProcessor, AppControllerEnable, NULL, UINT32_C(0));
    }
    if (RETCODE_OK != retcode)
    {
        printf("AppControllerSetup : Failed \r\n");
        Retcode_RaiseError(retcode);
        assert(0); /* To provide LED indication for the user */
    }
}

/* global functions ********************************************************* */

/** Refer interface header for description */
void AppController_Init(void * cmdProcessorHandle, uint32_t param2)
{
    BCDS_UNUSED(param2);

    Retcode_T retcode = RETCODE_OK;

    if (cmdProcessorHandle == NULL)
    {
        printf("AppController_Init : Command processor handle is NULL \r\n");
        retcode = RETCODE(RETCODE_SEVERITY_ERROR, RETCODE_NULL_POINTER);
    }
    else
    {
        AppCmdProcessor = (CmdProcessor_T *) cmdProcessorHandle;
        retcode = CmdProcessor_Enqueue(AppCmdProcessor, AppControllerSetup, NULL, UINT32_C(0));
    }

    if (RETCODE_OK != retcode)
    {
        Retcode_RaiseError(retcode);
        assert(0); /* To provide LED indication for the user */
    }
}

/**@} */
/** ************************************************************************* */
